/*
PROJETO: Game OPENGL "Batalha Naval"
Author	: Roger Larico Chavez
RA		: 022693

  FEEC UNICAMP
*/
#include "stdafx.h" 
#include <windows.h>
//Se nao e windows tire as duas linhas arriba

#include "glut.h"
#include <stdio.h>
#include <stdlib.h>



#define COLOR /255
#define PI 3.1415

#define COORD_TEXTURA_PLANO 1.0
#define COORD_TEXTURA_AVIAO 1.0
#define COR_DO_PLANO 0.52,0.52,0.78,1.0
#define COR_DO_AVIAO 0.3,0.52,0.18,1.0
#define TEXTURA_DO_PLANO "agua1.rgb"
#define TEXTURA_DO_AVIAO "camuflagem2.rgb"

GLint WIDTH =800;
GLint HEIGHT=600;

//GLfloat obs[3]={0.0,7.0,0.0};
//GLfloat look[3]={0.0,3.0,0.0};
GLuint  textura_plano;
GLuint  textura_aviao;

GLshort texturas=1;
GLfloat tetaxz=0;
GLfloat raioxz=6;

GLfloat ctp[4][2]={
  {-COORD_TEXTURA_PLANO,-COORD_TEXTURA_PLANO},
  {+COORD_TEXTURA_PLANO,-COORD_TEXTURA_PLANO},
  {+COORD_TEXTURA_PLANO,+COORD_TEXTURA_PLANO},
  {-COORD_TEXTURA_PLANO,+COORD_TEXTURA_PLANO}
};

GLfloat cta[4][2]={
  {-COORD_TEXTURA_AVIAO,-COORD_TEXTURA_AVIAO},
  {+COORD_TEXTURA_AVIAO,-COORD_TEXTURA_AVIAO},
  {+COORD_TEXTURA_AVIAO,+COORD_TEXTURA_AVIAO},
  {-COORD_TEXTURA_AVIAO,+COORD_TEXTURA_AVIAO}
};



typedef struct{
  unsigned short imagic;
  unsigned short type;
  unsigned short dim;
  unsigned short sizeX, sizeY, sizeZ;
  char name[128];
  unsigned char *data;
} IMAGE;

IMAGE *ImageLoad(char *);
#define IMAGIC      0x01da
#define IMAGIC_SWAP 0xda01

#define SWAP_SHORT_BYTES(x) ((((x) & 0xff) << 8) | (((x) & 0xff00) >> 8))
#define SWAP_LONG_BYTES(x) (((((x) & 0xff) << 24) | (((x) & 0xff00) << 8)) | \
((((x) & 0xff0000) >> 8) | (((x) & 0xff000000) >> 24)))

typedef struct{
  unsigned short imagic;
  unsigned short type;
  unsigned short dim;
  unsigned short sizeX, sizeY, sizeZ;
  unsigned long min, max;
  unsigned long wasteBytes;
  char name[80];
  unsigned long colorMap;
  FILE *file;
  unsigned char *tmp[5];
  unsigned long rleEnd;
  unsigned long *rowStart;
  unsigned long *rowSize;
} Image;


static Image *ImageOpen(char *fileName){
  Image *image;
  unsigned long *rowStart, *rowSize, ulTmp;
  int x, i;

  image = (Image *)malloc(sizeof(Image));
  if (image == NULL){
    fprintf(stderr, "Out of memory!\n");
    exit(-1);
  }
  if ((image->file = fopen(fileName, "rb")) == NULL){
    perror(fileName);
    exit(-1);
  }
  /*
   *	Read the image header
   */
  fread(image, 1, 12, image->file);
  /*
   *	Check byte order
   */
  if (image->imagic == IMAGIC_SWAP){
    image->type = SWAP_SHORT_BYTES(image->type);
    image->dim = SWAP_SHORT_BYTES(image->dim);
    image->sizeX = SWAP_SHORT_BYTES(image->sizeX);
    image->sizeY = SWAP_SHORT_BYTES(image->sizeY);
    image->sizeZ = SWAP_SHORT_BYTES(image->sizeZ);
  }

  for ( i = 0 ; i <= image->sizeZ ; i++ ){
    image->tmp[i] = (unsigned char *)malloc(image->sizeX*256);
    if (image->tmp[i] == NULL ) {
      fprintf(stderr, "Out of memory!\n");
      exit(-1);
    }
  }

  if ((image->type & 0xFF00) == 0x0100){ /* RLE image */
    x = image->sizeY * image->sizeZ * sizeof(long);
    image->rowStart = (unsigned long *)malloc(x);
    image->rowSize = (unsigned long *)malloc(x);
    if (image->rowStart == NULL || image->rowSize == NULL){
      fprintf(stderr, "Out of memory!\n");
      exit(-1);
    }
    image->rleEnd = 512 + (2 * x);
    fseek(image->file, 512, SEEK_SET);
    fread(image->rowStart, 1, x, image->file);
    fread(image->rowSize, 1, x, image->file);
    if (image->imagic == IMAGIC_SWAP){
      x /= sizeof(long);
      rowStart = image->rowStart;
      rowSize = image->rowSize;
      while (x--){
	ulTmp = *rowStart;
	*rowStart++ = SWAP_LONG_BYTES(ulTmp);
	ulTmp = *rowSize;
	*rowSize++ = SWAP_LONG_BYTES(ulTmp);
      }
    }
  }
  return image;
}

static void ImageClose( Image *image){
  int i;

  fclose(image->file);
  for ( i = 0 ; i <= image->sizeZ ; i++ )
    free(image->tmp[i]);
  free(image);
}

static void ImageGetRow( Image *image, unsigned char *buf, int y, int z){
  unsigned char *iPtr, *oPtr, pixel;
  int count;

  if ((image->type & 0xFF00) == 0x0100){  /* RLE image */
    fseek(image->file, image->rowStart[y+z*image->sizeY], SEEK_SET);
    fread(image->tmp[0], 1, (unsigned int)image->rowSize[y+z*image->sizeY],
	  image->file);

    iPtr = image->tmp[0];
    oPtr = buf;
    while (1){
      pixel = *iPtr++;
      count = (int)(pixel & 0x7F);
      if (!count)
	return;
      if (pixel & 0x80){
	while (count--){
	  *oPtr++ = *iPtr++;
	}
      } 
      else{
	pixel = *iPtr++;
	while (count--){
	  *oPtr++ = pixel;
	}
      }
    }
  }
  else{ /* verbatim image */
    fseek(image->file, 512+(y*image->sizeX)+(z*image->sizeX*image->sizeY),
	  SEEK_SET);
    fread(buf, 1, image->sizeX, image->file);
  }
}

static void ImageGetRawData( Image *image, char *data)
{
  int i, j, k;
  int remain;

  switch ( image->sizeZ ){
  case 1:
    remain = image->sizeX % 4;
    break;
  case 2:
    remain = image->sizeX % 2;
    break;
  case 3:
    remain = (image->sizeX * 3) & 0x3;
    if (remain)
      remain = 4 - remain;
    break;
  case 4:
    remain = 0;
    break;
  }

  for (i = 0; i < image->sizeY; i++) {
    for ( k = 0; k < image->sizeZ ; k++ )
      ImageGetRow(image, image->tmp[k+1], i, k);
    for (j = 0; j < image->sizeX; j++) 
      for ( k = 1; k <= image->sizeZ ; k++ )
	*data++ = *(image->tmp[k] + j);
    data += remain;
  }
}

IMAGE *ImageLoad(char *fileName)
{
  Image *image;
  IMAGE *final;
  int sx;

  image = ImageOpen(fileName);

  final = (IMAGE *)malloc(sizeof(IMAGE));
  if (final == NULL){
    fprintf(stderr, "Out of memory!\n");
    exit(-1);
  }
  final->imagic = image->imagic;
  final->type = image->type;
  final->dim = image->dim;
  final->sizeX = image->sizeX; 
  final->sizeY = image->sizeY;
  final->sizeZ = image->sizeZ;

  /* 
   * Round up so rows are long-word aligned 
   */
  sx = ( (image->sizeX) * (image->sizeZ) + 3) >> 2;

  final->data 
    = (unsigned char *)malloc( sx * image->sizeY * sizeof(unsigned int));

  if (final->data == NULL){
    fprintf(stderr, "Out of memory!\n");
    exit(-1);
  }

ImageGetRawData(image, (char *)final->data);
ImageClose(image);
return final;
}
///////////////////////////////////
// DECLARACION DE LA CLASE CTextura
///////////////////////////////////

#include <stdio.h>

class CTextura
{
	public : 
		// Constructor
		CTextura(void);

		// Habilita las texturas
		static void HabilitarTexturas(void);

		// Leer la textura desde un archivo BMP de Windows de 24 bits
		// Devuelve 1 si se ha leido la textura
		// Devuelve 0 en otro caso
		int LeerTextura(const char *Archivo);

		// Pone a textura como la activa de OpenGL
		void PonerActiva(void);
		
		// Destructor
		~CTextura();

		// Puntero na imagen da textura
		unsigned char *Imagen;

		// Tamanho da textura 
		int TamX,TamY;
};

///////////////////////////////////
// IMPLEMENTACION DE LA CLASE CTextura
///////////////////////////////////

CTextura :: CTextura(void)
{
	Imagen = NULL;
}

int CTextura :: LeerTextura(const char *Textura)
{
	BOOL b = FALSE;

	int namelen = strlen(Textura);
	if ( namelen > 4 )
	{
		char suffix[5];
		strncpy(suffix, Textura + namelen - 4, 4);
		suffix[4] = 0;
		_strupr(suffix);

		// is it a BMP?
		if ( !strcmp(suffix, ".BMP") )
		{
			FILE* file = fopen(Textura, "rb");
			if ( file != NULL )
			{
				BITMAPFILEHEADER fileheader;
				BITMAPINFOHEADER infoheader;

				if ( fread(&fileheader, sizeof(BITMAPFILEHEADER), 1, file) == 1
					&& (char)fileheader.bfType == 'B' && *(((char*)&fileheader.bfType) + 1) == 'M'
					&& fread(&infoheader, sizeof(BITMAPINFOHEADER), 1, file) == 1
					&& infoheader.biBitCount == 24 && infoheader.biCompression == BI_RGB )
				{
					
					rewind( file );
					long offset = fileheader.bfOffBits;
					fseek(file, offset, SEEK_SET);

					int bufsize = infoheader.biWidth * abs(( int ) infoheader.biHeight) * 3;
					unsigned char* imagebuf = new unsigned char[bufsize];

					if ( (int)fread(imagebuf, 1, bufsize, file) == bufsize )
					{
						// Flip the RGB components
						for ( int i = 0; i < bufsize / 3; ++i )
						{
							unsigned char c = imagebuf[ i * 3 ];
							imagebuf[ i * 3 ] = imagebuf[ ( i * 3 ) + 2 ];
							imagebuf[ ( i * 3 ) + 2 ] = c;
						}

						//AbVec2s size((short)infoheader.biWidth, (short)abs((int)infoheader.biHeight));
						TamX = infoheader.biWidth;
						TamY = infoheader.biHeight;
						//image.setValue(size, 3, imagebuf);
						b = TRUE;
					}

					//delete [] imagebuf;
					Imagen = imagebuf;
				}

				fclose(file);
			}
		}
	}

	return b;
}

void CTextura :: PonerActiva(void)
{
	if(Imagen)
		glTexImage2D(GL_TEXTURE_2D,0,3,TamX, TamY,0,GL_RGB,GL_UNSIGNED_BYTE, Imagen);
}

void CTextura :: HabilitarTexturas(void)
{
	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glEnable(GL_TEXTURE_2D);
}

CTextura :: ~CTextura()
{
	if(Imagen)
		delete Imagen;
}

/////////////////////////////////////////////////
// Variables globales :

CTextura *Textura;

/*fin TExTure*/

struct Nave
{
	int		type;
	char*	descrip;
	int		large;
	int		position[4][2];
}NaveA,NaveB,NaveC;

float random(float ini, float end)
{
	float rnd,r2;
	r2=rand();
	rnd=abs((end-ini)*r2/(RAND_MAX)+ini);	
	return (rnd);
}

// Angulos
float angle, angle2,angle3;

int moving, startx, starty,nomodel, flag=1;
   

GLfloat  ambient_light[] = {0.3, 0.3, 0.45, 1.0};
GLfloat   source_light[] = {0.9, 0.8, 0.8, 1.0};
GLfloat      light_pos[] = {7.0, 0.0, 0.0, 1.0};



GLfloat     spec_light[] = { 1.0, 1.0, 1.0, 1.0};


GLfloat  ambient_light_n []	= { 0.0, 0.0, 0.0, 1.0 }; 
GLfloat   source_light_n []	= { 0.2, 0.2, 0.2, 1.0 }; 
GLfloat     spec_light_n []	= { 1.0, 1.0, 1.0, 1.0};

GLfloat mat_ambient_diffuse [] = {0.0,0.0,1.0,1.0};
GLfloat mat_specular []		= {0.7,0.0,0.0,1.0};
GLfloat mat_emission []		= {0.0,0.0,0.0,1.0};
GLfloat mat_shininess		= 0.2;

GLfloat spot_dir [] = {0.0,0.0,-1.0};

GLfloat focus_emission [] = {0.8,0.8,0.8,1.0};


 
void DisplayListNaveA()
{	
  GLUquadricObj* q;
  glNewList(NaveA.large , GL_COMPILE);
  q = gluNewQuadric();

  
	glPushMatrix();
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glClearDepth(1.0);
	glTranslatef(0,5,1);
	glRotatef(90, 1, 0, 0);  
	//glColor3f(0.70,0.6,0.72);
//	glColor3f(0.10,0.10,0.72);		
	glPushMatrix();
 	glBegin(GL_TRIANGLES);			
			glTexCoord2f(1.0,1.0);glVertex3f(15,	-2.2,	7.5);
			glTexCoord2f(1.0,0.0);glVertex3f(23,	2.8,	0);	
			glTexCoord2f(0.0,1.0);glVertex3f(15,	2.8,	7.5);
			
			glTexCoord2f(1.0,1.0);glVertex3f(15,	2.8,	-7.5);					
			glTexCoord2f(1.0,0.0);glVertex3f(23,	2.8,	0);	
			glTexCoord2f(0.0,1.0);glVertex3f(15,	-2.2,	-7.5);

			glTexCoord2f(1.0,1.0);glVertex3f(15,	2.8,	7.5);				
			glTexCoord2f(1.0,0.0);glVertex3f(23,	2.8,	0);	
			glTexCoord2f(0.0,1.0);glVertex3f(15,	2.8,	-7.5);
			/**/
			glTexCoord2f(1.0,1.0);glVertex3f(23,	2.8,	0);	
			glTexCoord2f(1.0,0.0);glVertex3f(15,	-2.2,	7.5);
			glTexCoord2f(0.0,1.0);glVertex3f(15,	-2.2,	-7.5);					
	glEnd();
  glPopMatrix();

  glDisable(GL_TEXTURE_2D); 

  /**/ 
//glColor3f(0.10,0.6,0.2);
  glPushMatrix();
  
  glTranslatef(-5,15,0);
	glBegin(GL_LINES);
	//glColor3f(0.10,0.6,0.2);
		glVertex3f(0,	-5,	0);		
		glVertex3f(0,	0,	0);		
	glEnd();

  float rnd1=random(0,90);
  glRotatef(rnd1, 0, 1, 0);
  gluCylinder(q,0.1,3,2,8,8);
  glPopMatrix();
/**/
  //glColor3f(1,0,1);
  glPushMatrix();
  glTranslatef(-8.3,8.5,4);
  glRotatef(90, 0, 1, 0);
  gluCylinder(q,1,1.3,2,8,8);  
  glRotatef(180, 0, 1, 0);
  gluCylinder(q,1,1.3,2,8,8);
  glPopMatrix();
  glPushMatrix();
  glTranslatef(-8.3,8.5,-4);
  glRotatef(90, 0, 1, 0);
  gluCylinder(q,1,1.3,2,8,8);
  glRotatef(180, 0, 1, 0);
  gluCylinder(q,1,1.3,2,8,8);
  glPopMatrix();
/**/
  //glColor3f(0.50,0.6,0.52);  
  glPushMatrix();
  glScalef(6,1,3);
  glTranslatef(0,0.35,0);
  glutSolidCube(5);
  glPopMatrix();
 

/**/
  //glColor3f(0,0.2,0.1);
  glPushMatrix();
  glScalef(4,0.5,3);
  glTranslatef(-1,8.35,0);
  //glTexCoord2f(1.0,1.0);
  glutSolidCube(5);
  glPopMatrix();

/**/
  
  glColorMaterial(GL_FRONT,GL_EMISSION);
  glEnable(GL_COLOR_MATERIAL);
  glColor4fv(focus_emission);
  

  glPushMatrix();
  glScalef(1,1,1);
  glTranslatef(-6,8.15,0);
  glutSolidCube(5);
  glPopMatrix();
  //glColor3f(1,1,1);
  
  glColor4fv(mat_emission);
  glDisable(GL_COLOR_MATERIAL);

glPopMatrix();
  gluDeleteQuadric(q);
  glEndList();

  //glColor3f(0,0,0);
}

void DisplayListNaveB()
{
	glDisable(GL_TEXTURE_2D); 
  GLUquadricObj* q;
  glNewList(NaveB.large, GL_COMPILE);
  q = gluNewQuadric();
  
  glTranslatef(0,15,10);
  glRotatef(90, 1, 0, 0);  

glPushMatrix();
    glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glClearDepth(1.0);
glDisable(GL_TEXTURE_2D); 

  glPushMatrix();
  glScalef(2,1,2);
  glRotatef(-45,0,1,0);
  glTranslatef(7.02,0,0.05);
  glutSolidCube(3);
  glPopMatrix();
/**/
  glPushMatrix();
  glScalef(6.8,1,2.77);
  glTranslatef(-0.01,0,3.62);
  glutSolidCube(3);
  glPopMatrix();
/**/
  glPushMatrix();
  glScalef(2,1,2);
  glTranslatef(1,3,5);
  glRotatef(-45,0,1,0);  
  glutSolidCube(3);
  glPopMatrix();
  /**/
  glPushMatrix();
  glTranslatef(-4,6,10);
  glScalef(0.15,0.15,0.15);
  glRotatef(-90, 1, 0, 0);
  glCallList(7);  //tower
  glPopMatrix();

  glPushMatrix();
  glScalef(3.8,1,2.77);
  glTranslatef(-1,3,3.62);
  glutSolidCube(3);
  glPopMatrix();
  /**/
  glPushMatrix();
  glScalef(3.8,1,2.77);
  glTranslatef(-1,5.8,3.62);
  glutSolidCube(2);
  glPopMatrix();

  /**/
 glPushMatrix();
 glTranslatef(4,3,8);
 glRotatef(90, 0, 1, 0);
 gluCylinder(q,0.5,0.5,2.5,20,10);
 glPopMatrix();
  /**/
 glPushMatrix();
 glTranslatef(4,3,12);
 glRotatef(90, 0, 1, 0);
 gluCylinder(q,0.5,0.5,2.5,20,10);
 glPopMatrix();
/**/
/**///janela lampada 
 glPushMatrix();
 glColorMaterial(GL_FRONT,GL_EMISSION);
 glEnable(GL_COLOR_MATERIAL);
 glColor4fv(focus_emission);
 glTranslatef(0,6,10);
 glRotatef(90, 0, 1, 0);
 glTranslatef(-1.5,0,0);
 glutSolidCube(1);
 glTranslatef(1.5,0,0);
 glutSolidCube(1);
 glTranslatef(1.5,0,0);
 glutSolidCube(1);
 glColor4fv(mat_emission);
 glDisable(GL_COLOR_MATERIAL);

  glPopMatrix();
 
 glDisable(GL_DEPTH_TEST);
glPopMatrix();
  gluDeleteQuadric(q);
  glEndList();
  glDisable(GL_TEXTURE_2D); 
}

void DisplayTower(void)
{

/**/
  GLUquadricObj* q;
  glNewList(7, GL_COMPILE);
  q = gluNewQuadric();
  glPushMatrix();

		glDepthFunc(GL_LEQUAL);
		glEnable(GL_DEPTH_TEST);
		glClearDepth(1.0);
			 //glColor3f(0,1,0);
		  glPushMatrix();
			  gluCylinder(q, 3, 0.5, 20, 12, 3);			  
		  glPopMatrix();
		  glPushMatrix();
			glRotatef(90, 1, 0, 0);
			glTranslatef(0,20,0);
			gluCylinder(q,0.1,10,5,8,8);
		  glPopMatrix();
		    glColorMaterial(GL_FRONT,GL_EMISSION);
			glEnable(GL_COLOR_MATERIAL);
			glColor4fv(focus_emission);

		  glPushMatrix();
			glTranslatef(0,-5,20);
			glRotatef(90, 1, 0, 0);
			gluCylinder(q,10,0.1,0.1,8,8);
		  glPopMatrix();
		  
		  glColor4fv(mat_emission);
		  glDisable(GL_COLOR_MATERIAL);

  glPopMatrix();
  gluDeleteQuadric(q);
  glEndList();
	
/**/		
}

void DisplayListNaveC()
{
	int i;
	GLUquadricObj* q;
	glNewList(NaveC.large, GL_COMPILE);
	q = gluNewQuadric();
	glTranslatef(0,15,10);
	glRotatef(90, 1, 0, 0);  
		
	glPushMatrix();
		glDepthFunc(GL_LEQUAL);
		glEnable(GL_DEPTH_TEST);
		glClearDepth(1.0);

		/**/
		glPushMatrix();
		glScalef(2,1,2);
		glRotatef(45,0,1,0);
		glTranslatef(-7,0,0);
		glutSolidCube(3);
		glPopMatrix();

		/**/
		glPushMatrix();
		glScalef(2,1,2);
		glRotatef(-45,0,1,0);
		glTranslatef(7,0,0);
		glutSolidCube(3);
		glPopMatrix();
		/**/

		glPushMatrix();
			glTranslatef(0,2.2,12.5);
			glScalef(0.4,0.4,0.4);
			glRotatef(135, 0, 1, 0);  
			glCallList(6);  //Jet
			glTranslatef(-5,0,5);
			glCallList(6);  //Jet
			glTranslatef(-5,0,5);
			glCallList(6);  //Jet
			glTranslatef(-5,0,5);
			glCallList(6);  //Jet
			glTranslatef(-5,0,5);
			glCallList(6);  //Jet
			glTranslatef(-5,0,5);
			glCallList(6);  //Jet
		glPopMatrix();

		glPushMatrix();
			glTranslatef(-10,4,12);
			glScalef(0.1,0.1,0.1);
			glRotatef(-90, 1, 0, 0);
			glCallList(7);  //tower
		glPopMatrix();

		glPushMatrix();
			glTranslatef(-12,4,10);
			glScalef(0.05,0.05,0.05);
			glRotatef(-90, 1, 0, 0);
			glRotatef(-90, 0, 0, 1);
			glCallList(7);  //tower
		glPopMatrix();

		glPushMatrix();
			glTranslatef(-8,4,10);
			glScalef(0.07,0.07,0.07);
			glRotatef(-90, 1, 0, 0);
			glRotatef(+90, 0, 0, 1);
			glCallList(7);  //tower
		glPopMatrix();

		glPushMatrix();
			glTranslatef(19,3,8);
			glScalef(0.4,0.4,0.4);
			glRotatef(90, 0, 1, 0);  
			glRotatef(-25, 1, 0, 0);  
			glCallList(6);  //Jet
		glPopMatrix();

		/**/
		glPushMatrix();
		glScalef(6.6,1,2.76);
		glTranslatef(0,0,3.62);
		glutSolidCube(3);
		glPopMatrix();
		
		/*luz pista: centro*/
		glPushMatrix();
		glScalef(0.5,0.2,0.2);
		glTranslatef(-11,9,40);


		for(i=0;i<16;i++)
		{
			glColorMaterial(GL_FRONT,GL_EMISSION);
			glEnable(GL_COLOR_MATERIAL);
			glColor4fv(focus_emission);

			glTranslatef(3,0,0);
			glutSolidCube(2);			
		}	
		glPopMatrix();
		/**/

		/*luz pista: lados*/
		glPushMatrix();	
		glTranslatef(-5,2,10);
		for(i=0;i<24;i++)
		{
			glTranslatef(1,0,0);
			glutSolidSphere(0.3,3,3);
			glPushMatrix();	
				glTranslatef(0,0,-4);
				glutSolidSphere(0.3,3,3);
			glPopMatrix();

		}	
		glPopMatrix();
		/**/
		glColor4fv(mat_emission);
		glDisable(GL_COLOR_MATERIAL);

		/*pista de avioes*/
		glPushMatrix();
		glScalef(11,0.2,2.76);
		glTranslatef(0,8,3.62);
		glutSolidCube(3);
		glPopMatrix();

		glPushMatrix();
		glScalef(11,0.2,1.76);
		glTranslatef(0.3,8,4.62);
		glutSolidCube(3);
		glPopMatrix();

		/**/
		glPushMatrix();
		glScalef(9.6,2,5.76);
		glTranslatef(-1,1.5,1.8);
		glutSolidCube(1);
		glPopMatrix();
		/**/

		glDisable(GL_DEPTH_TEST);
	glPopMatrix();	
	gluDeleteQuadric(q);
	glEndList();
}

void carregar_texturas(void)
{
  IMAGE *img;
  GLenum gluerr;

  /* textura do plano */
  glGenTextures(1, &textura_plano);
  glBindTexture(GL_TEXTURE_2D, textura_plano);
  
  if(!(img=ImageLoad(TEXTURA_DO_PLANO))) {
    fprintf(stderr,"Error reading a texture.\n");
    exit(-1);
  }

  gluerr=gluBuild2DMipmaps(GL_TEXTURE_2D, 3, 
			   img->sizeX, img->sizeY, 
			   GL_RGB, GL_UNSIGNED_BYTE, 
			   (GLvoid *)(img->data));
  if(gluerr){
    fprintf(stderr,"GLULib%s\n",gluErrorString(gluerr));
    exit(-1);
  }

  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);

  /* textura do aviao */
  glGenTextures(1, &textura_aviao);
  glBindTexture(GL_TEXTURE_2D, textura_aviao);

  
  if(!(img=ImageLoad(TEXTURA_DO_AVIAO))) {
    fprintf(stderr,"Error reading a texture.\n");
    exit(-1);
  }

  gluerr=gluBuild2DMipmaps(GL_TEXTURE_2D, 3, 
			   img->sizeX, img->sizeY, 
			   GL_RGB, GL_UNSIGNED_BYTE, 
			   (GLvoid *)(img->data));
  if(gluerr){
    fprintf(stderr,"GLULib%s\n",gluErrorString(gluerr));
    exit(-1);
  }

  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
  glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);
  
}

void DisplayListJet(void)
{
	GLint jetid=6;/*id do Jet*/
	GLUquadricObj *q;
	GLfloat asa[][3]={{-4.0,0.0,0.0},{+4.0,0.0,0.0},{0.0,0.0,3.0}};
	GLfloat cauda[][3]={{0.0,0.0,0.0},{0.0,2.0,-1.0},{0.0,2.0,0.0},{0.0,0.0,2.0}};
	

  /* inicia a composicao do JET */
	glNewList(jetid, GL_COMPILE);		
	glPushMatrix ( );		
		glDepthFunc(GL_LEQUAL);
		glEnable(GL_DEPTH_TEST);
		glClearDepth(1.0);
		glColorMaterial(GL_FRONT,GL_EMISSION);
			glEnable(GL_COLOR_MATERIAL);
			glColor4fv(focus_emission);
			
			q = gluNewQuadric();  
			glPushMatrix();
				glTranslatef(2,0,0);
				gluCylinder(q, 0.3, 0.1, 1.5, 12, 3);
				glTranslatef(-4,0,0);
				gluCylinder(q, 0.3, 0.1, 1.5, 12, 3);
			glPopMatrix();
			
			glColor4fv(mat_emission);
			glDisable(GL_COLOR_MATERIAL);
		/* asas */
		glBegin(GL_TRIANGLES); /*ambas caras*/
			glTexCoord2f(1.0,1.0);glVertex3fv(asa[0]);
			glTexCoord2f(1.0,0.0);glVertex3fv(asa[1]);
			glTexCoord2f(0.0,1.0);glVertex3fv(asa[2]);
			glTexCoord2f(1.0,1.0);glVertex3fv(asa[2]);
			glTexCoord2f(1.0,0.0);glVertex3fv(asa[1]);
			glTexCoord2f(0.0,1.0);glVertex3fv(asa[0]);
		glEnd();

			

		/* corpo */
		q = gluNewQuadric();  
		gluCylinder(q, 0.5, 0.5, 4, 12, 3);
		/* nariz */
		
		q = gluNewQuadric();
		glPushMatrix();
		glTranslatef(0,0,4);
		gluCylinder(q, 0.5, 0.0, 1.5, 12, 3);
		glPopMatrix();
		/* cauda */		

	    glBegin(GL_POLYGON); 
		  glTexCoord2fv(cta[0]); glVertex3fv(cauda[0]);
		  glTexCoord2fv(cta[1]); glVertex3fv(cauda[1]);
		  glTexCoord2fv(cta[2]); glVertex3fv(cauda[2]);
		  glTexCoord2fv(cta[3]); glVertex3fv(cauda[3]);
		  glEnd();

		glBegin(GL_POLYGON); 
			glTexCoord2f(1.0,1.0);glVertex3fv(cauda[3]);
			glTexCoord2f(1.0,0.0);glVertex3fv(cauda[2]);
			glTexCoord2f(0.0,1.0);glVertex3fv(cauda[1]);
			glTexCoord2f(0.0,1.0);glVertex3fv(cauda[0]);
		glEnd();
		/* cabine do piloto */	  
		glTranslatef(0,0.3,3.5);
		glPushMatrix();
			
			glColorMaterial(GL_FRONT,GL_EMISSION);
			glEnable(GL_COLOR_MATERIAL);
			glColor4fv(focus_emission);
	

			glScalef(0.7,0.7,2.0);
			q=gluNewQuadric();
			glColor3f(0.3,0.5,1);
			 glDisable(GL_TEXTURE_2D);
			gluSphere(q,0.5,12,12);
			
			glColor4fv(mat_emission);
			glDisable(GL_COLOR_MATERIAL);

			/* cabine do piloto */
  
		glPopMatrix();
	glPopMatrix();
  glEndList();
  glDisable(GL_TEXTURE_2D);

}

void DisplayListGlobe( void )
{	
  GLUquadricObj* q;
  glNewList(5 , GL_COMPILE);
  q = gluNewQuadric();
  glPushMatrix ( );
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glClearDepth(1.0);
  glPushMatrix ( );
   glRotatef ( 90.0, 1.0, 0.0, 0.0 );   
   gluQuadricNormals ( q, GLU_SMOOTH );   
   gluSphere ( q, 6.0, 20, 20 );
  glPopMatrix ( );  
  /*
  glPushMatrix();
	glScalef(11,0.2,2.76);
	glTranslatef(0,10,3.62);
	glutSolidCube(3);
  glPopMatrix();
  /**/
  glPopMatrix();
  gluDeleteQuadric(q);
  glEndList();   
}



void init ( void )
{	
	carregar_texturas();
	nomodel=2;

	glClearColor(0.0,0.0,0.0,0.0);
	glDepthFunc(GL_LEQUAL);
	glShadeModel(GL_FLAT);

	
   // Set up de modelo de iluminacaao simples
   glEnable ( GL_LIGHTING );
   glLightModelfv ( GL_LIGHT_MODEL_AMBIENT, ambient_light );
   glLightfv (GL_LIGHT0,GL_DIFFUSE, source_light );
   glLightfv ( GL_LIGHT0,GL_POSITION, light_pos  );
   glEnable  ( GL_LIGHT0 );

   // Enable de las propiedades del material para la iluminacin
   glEnable ( GL_COLOR_MATERIAL );
   glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT1);


   
	 /**/
   
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glClearDepth(1.0);

   glEnable ( GL_CULL_FACE );
   /**/
   //glClearColor (0.0, 0.0, 0.0, 0.0);


   /*TextuRE*
   // Creamos y cargamos la textura
	Textura = new CTextura;
	if(!Textura->LeerTextura("text1.bmp"))
	{
		printf("erro  bitmap\n");
		exit(-1);
	}
	CTextura :: HabilitarTexturas();
	Textura->PonerActiva();

   /*fin TExtuRe*/




  //glEnable(GL_TEXTURE_2D);

   
 
	glTranslatef(0, 0, -50); //z PARA ATRAS
	NaveA.large=2;
	NaveB.large=3;
	NaveC.large=4;
	//displaylist
	DisplayListNaveA();//2
	DisplayListNaveB();//3
	DisplayListNaveC();//4
	DisplayListGlobe();//5
	DisplayListJet();//6
	DisplayTower();//7


}

void display ( void )
{
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// no es necesario tener control de profundidad...
   glClear   ( GL_COLOR_BUFFER_BIT );
   glLightfv ( GL_LIGHT0,GL_POSITION, light_pos );
   //glLightfv ( GL_LIGHT1,GL_POSITION, light_pos2 );
   //glLightfv ( GL_LIGHT2,GL_POSITION, light_pos3 );
   
   glPushMatrix ( );
   glTranslatef ( 0.0, 0.0, -50.0 );
   glRotatef    ( angle2, 1.0, 0.0, 0.0 );
   glRotatef    ( angle, 0.0, 1.0, 0.0  );
   glRotatef    ( angle3, 0.0, 0.0, 1.0  );

   //glScalef    ( 10, 10, 10 );

   /* habilita/desabilita uso de texturas*/
  if(texturas){
    glEnable(GL_TEXTURE_2D);  
  }
  else{
    glDisable(GL_TEXTURE_2D);
  }

  if(nomodel==6){

  glColor4f(COR_DO_PLANO);
  glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_DECAL);
  glBindTexture(GL_TEXTURE_2D,textura_plano);
   
  glBegin(GL_QUADS);
  glTexCoord2fv(ctp[0]);  glVertex3f(-10,0,10);
  glTexCoord2fv(ctp[1]);  glVertex3f(10,0,10);
  glTexCoord2fv(ctp[2]);  glVertex3f(10,0,-10);
  glTexCoord2fv(ctp[3]);  glVertex3f(-10,0,-10);
  glEnd();

  glTranslatef(0.0,2.0,-3.0);
  }

  glColor4f(COR_DO_AVIAO);
  glBindTexture(GL_TEXTURE_2D,textura_aviao);
     
   glCallList(nomodel);


   //glutSwapBuffers();


   glPopMatrix  ( );
   glutSwapBuffers( );
}

void reshape ( int w, int h )
{
   glViewport   ( 0, 0, (GLint) w, (GLint) h );
   glMatrixMode ( GL_PROJECTION);
   glLoadIdentity( );
   if ( h==0 )
      gluPerspective ( 45, (GLdouble)w, 1.0, 2000.0 );
   else
      gluPerspective ( 45, (GLdouble)w / (GLdouble)h,1.0, 2000.0 );

   glMatrixMode ( GL_MODELVIEW );
   glLoadIdentity ( );
}

void mouse ( int button, int state, int x, int y )
{
   if ( button == GLUT_LEFT_BUTTON && state == GLUT_DOWN )
   {
      moving = 1;
      startx = x;
      starty = y;
   }

   if ( button == GLUT_LEFT_BUTTON && state == GLUT_UP  )
      moving = 0;
}

void motion ( int x, int y )
{
   if ( moving )
   {
      angle = angle +   ( x - startx );
      angle2 = angle2 + ( y - starty );
      startx = x;
      starty = y;
      glutPostRedisplay ( );
   }
}

void idle_func ( void )
{
   //angle = angle + 0.1;//rotacao automatic
   if ( angle == 360 )
      angle = 0;
   glutPostRedisplay ( );
}



void keyboard ( unsigned char key, int x, int y )
{
   switch ( tolower(key) )
   {
      case 27:  /*  Escape = Sair*/
         exit(0);
         break;
      case 'f':
        glutFullScreen ( );
        break;
      case 'w':  
		 glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);		
         break;
	  case 'z':
         angle3++;
         break;
	  case 's':  
		  glPolygonMode(GL_FRONT_AND_BACK, GL_FILL );				 
         break;
	  case 'i': 
		  if(flag==1)
			glDisable( GL_LIGHTING );
		  else
			glEnable ( GL_LIGHTING );
		  flag=(-1*flag);
         break;
	  case 'a':
         nomodel=2;
         break;
	  case 'b':
         nomodel=3;
         break;
	  case 'c':
         nomodel=4;
         break; 
	  case 'd':
         nomodel=5;
         break;
	  case 'e':
         nomodel=6;
         break;
	  case 'g':
         nomodel=7;
         break;
	  case '+':
         glTranslatef(0,0,1);
         break;
	  case '-':
         glTranslatef(0,0,-1);
		 break;
	  case 52:/*esq x*/
         glTranslatef(-1,0,0);
		 break;
	  case 54:/*dire x*/
         glTranslatef(1,0,0);
		 break;
	  case 50:/*down y */
         glTranslatef(0,-1,0);
		 break;
	  case 56:/*up y */
         glTranslatef(0,+1,0);
		 break;
      default:
         break;
   }
}

/*  Bucle principal */

int main ( int argc, char** argv )
{
   glutInit ( &argc, argv );
   glutInitDisplayMode( GLUT_RGB | GLUT_DOUBLE );
   glutInitWindowSize (600,400);
   glutCreateWindow ("Model OpenGL: Batalha Naval");   
   init();
   glutReshapeFunc  ( reshape   );
   glutMouseFunc    ( mouse     );
   glutMotionFunc   ( motion    );
   glutKeyboardFunc ( keyboard  );
   glutDisplayFunc  ( display   );
   glutIdleFunc     ( idle_func );
   glutMainLoop ( );
   return 0;
}